<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    <link href="https://fonts.googleapis.com/css2?family=Outfit:wght@100..900&display=swap" rel="stylesheet">
</head>
<body style="margin: 0; background-color: #f9f9f9; font-family: 'Outfit', 'sans-serif'">
<table border="0" cellpadding="0" cellspacing="0" width="100%"
       style="margin: 0; padding: 0; border: 0; width: 100%;">
    <tr>
        <td style="padding: 0; margin: 0;">
            <table border="0" cellpadding="0" cellspacing="0"
                   style="margin: 16px auto; border: 0; width: 100%; max-width: 420px; padding: 0;">
                <tr>
                    <td>
                        <table border="0" cellpadding="0" cellspacing="0"
                               style="border: 0; width: 100%;">
                            <tr>
                                <td align="center" style="padding: 0; margin: 0 auto;">
                                    <img style="width: 122px; height: 23px;"
                                         src="{{ IMAGE_LOGO.src }}"
                                         alt="">
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
    <tr>
        <td style="padding: 0 0 16px 0; margin: 0;">
            <table border="0" cellpadding="0" cellspacing="0"
                   style="box-shadow: 5px 5px 10px 0 #4F4F4F0D, -3px -3px 10px 0 #4F4F4F0D; margin: 0 auto; border: 0; width: 100%; border-radius: 12px; max-width: 420px; padding: 0; background-color: #FFFFFF;">
                <tr>
                    <td style="padding:0; margin:0;">
                        <table border="0" cellpadding="0" cellspacing="0"
                               style="border: 0; padding: 0; margin: 40px 40px 0; max-width: 370px;">
                            <tr>
                                <td style="padding: 0 0 20px; margin: 0; font-size: 20px; line-height: 24px; font-weight: 600; color: #262730">
                                    You have a new message...
                                    <img style="margin: 0 0 -3px 6px; width: 20px; height: 20px;"
                                         src="{{ IMAGE_MAIL.src }}"
                                         alt="">
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
                <tr>
                    <td style="padding:0; margin:0;">
                        <table border="0" cellpadding="0" cellspacing="0"
                               style="border: 0; padding: 0; margin: 0 40px; max-width: 370px;">
                            <tr>
                                <td style="text-align: left; padding: 0 0 20px; margin: 0; font-size: 14px; line-height: 24px; font-weight: 400; color: #262730">
                                    <b>$$USER_NAME$$</b> has just sent you a message on Leadguru
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
                <tr>
                    <td style="padding:0; margin:0;">
                        <table border="0" cellpadding="0" cellspacing="0"
                               style="border: 0; padding: 0; margin: 0 40px; width: 100%;">
                            <tr>
                                <td style="padding: 0 0 20px; margin: 0; font-size: 14px; color: #434a57;font-weight: 600;">
                                    <a href="$$PORTAL_LINK$$/feed"
                                       target="_blank"
                                       style="margin: 0;
                                              width: 157px;
                                              display: flex;
                                              color: #ffffff;
                                              font-size: 14px;
                                              font-weight: 600;
                                              line-height: 20px;
                                              border-radius: 6px;
                                              text-decoration: none;
                                              box-sizing: border-box;
                                              background-color: #3579F6;
                                              padding: 8px 18px 8px 20px;">View message
                                        <img style="margin: 2px 0 0 12px; width: 16px; height: 16px;"
                                             src="{{ IMAGE_ARROW.src }}"
                                             alt="">
                                    </a>
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
                <tr>
                    <td>
                        <table border="0" cellpadding="0" cellspacing="0"
                               style="border: 0; margin: 0 40px; padding-bottom: 40px">
                            <tr>
                                <td style="padding: 20px 0 0; border-top: 1px solid #E9EBF0; margin: 0; font-size: 14px; line-height: 20px; color: #778198; text-align: left; font-weight: 400">
                                    Need help? Check out our
                                    <a style="color: #3579F6; text-decoration: none;"
                                       href="https://leadguru.co/features/"
                                       target="_blank" >Knowledge Base</a>
                                    or book $$HELP_TEXT$$
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
    <tr>
        <td style="padding: 0; margin: 0;">
            <table border="0" cellpadding="0" cellspacing="0"
                   style="margin: 0 auto; border: 0; width: 100%; max-width: 420px; padding: 0;">
                <tr>
                    <td>
                        <table border="0" cellpadding="0" cellspacing="0"
                               style="border: 0; width: 100%;">
                            <tr>
                                <td align="center" style="padding: 0; font-size: 14px; margin: 0; color: #778198; line-height: 20px">&copy;  2024 Leadguru. Bagno 2, Warsaw, Poland, 00-112</td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
</table>
</body>
</html>
